﻿using System.Windows;
using System.Windows.Input;
using System.Windows.Controls;
using System;
using System.Windows.Interop;

namespace MagicCat
{
    /// <summary>
    /// MainWindow.xaml 的交互逻辑
    /// </summary>
    public partial class MainWindow : Window
    {
        private readonly AppContext _appContext = AppContext.GetAppContext();
        private readonly SerialTool _serialTool = SerialTool.GetSerialTool();

        private bool _noDevice;

        public static Label Status;
        public static Label FaceName;
        public static Image FaceSrc;

        public MainWindow()
        {
            InitializeComponent();

            Status = PortStatus;
            FaceName = FileName;
            FaceSrc = Face;
            LoadDevice();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            // 监听 Windows 消息，获取窗口句柄一定要写在窗口 loaded 事件里，才能获取到窗口句柄，否则为空
            // 窗口过程 - 挂钩
            (PresentationSource.FromVisual(this) as HwndSource)?.AddHook(DeviceChanged);
        }

        private void Window_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.Key)
            {
                case Key.Escape:
                    ToolBar.Visibility = Visibility.Visible;
                    Face.Margin = new Thickness { Left = 0, Top = 32, Right = 0, Bottom = 0 };
                    WindowState = WindowState.Normal;
                    WindowStyle = WindowStyle.SingleBorderWindow;
                    break;
            }
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            if (_noDevice) return;
            var messageBoxResult = MessageBox.Show("确定退出吗？", "提示", MessageBoxButton.OKCancel, MessageBoxImage.Question);
            if (messageBoxResult == MessageBoxResult.Cancel)
            {
                e.Cancel = true;
            }
            else
            {
                Application.Current.Shutdown();
            }
        }

        private void Settings_Click(object sender, RoutedEventArgs e)
        {
            new SettingsWindow().ShowDialog();
        }

        private void FullScreen_Click(object sender, RoutedEventArgs e)
        {
            ToolBar.Visibility = Visibility.Hidden;
            Face.Margin = new Thickness { Left = 0, Top = 0, Right = 0, Bottom = 0 };
            this.Topmost = false;
            this.WindowStyle = WindowStyle.None;
            this.WindowState = WindowState.Maximized;
        }

        private void LoadDevice()
        {
            var portList = _serialTool.FindPort();
            MessageBoxResult mbr;
            switch (portList.Count)
            {
                case 0:
                    mbr = MessageBox.Show("没有找到可用硬件！\n请确保连接硬件后再打开本软件。", "错误提示", MessageBoxButton.OK, MessageBoxImage.Warning);
                    if (mbr == MessageBoxResult.OK)
                    {
                        _noDevice = true;
                        Close();
                    }
                    break;
                case 1:
                    var serialPort = _serialTool.OpenPort(portList[0], "9600", "8", "1", "无", _appContext.DataReceived);
                    if (serialPort != null)
                    {
                        _appContext.SerialPort = serialPort;
                    }
                    break;
                default:
                    mbr = MessageBox.Show("请到设置页面配置正确串口！", "提示", MessageBoxButton.OK, MessageBoxImage.Information);
                    if (mbr == MessageBoxResult.OK)
                    {

                    }
                    break;
            }
        }

        // Windows 消息编号，采用 Windows 的消息机制来捕获插入的 USB 状态
        public const int WmDeviceChange = 0x219;
        public const int DbtDeviceArrival = 0x8000;
        public const int DbtDeviceRemoveComplete = 0x8004;

        // 设备插拔改变函数
        private IntPtr DeviceChanged(IntPtr hwnd, int msg, IntPtr wParam, IntPtr lParam, ref bool handled)
        {
            if (msg == WmDeviceChange)
            {
                switch (wParam.ToInt32())
                {
                    case DbtDeviceArrival:
                        // 设备插入
                        LoadDevice();
                        Console.WriteLine("设备插入");
                        break;
                    case DbtDeviceRemoveComplete:
                        // 设备卸载
                        Console.WriteLine("设备卸载");
                        break;
                }
            }
            return IntPtr.Zero;
        }
    }
}
