﻿using System;
using System.Collections.Generic;
using System.IO.Ports;

namespace MagicCat
{
    class SerialTool
    {
        private static SerialTool serialTool = null;

        private SerialTool() { }

        public static SerialTool GetSerialTool()
        {
            return serialTool ?? (serialTool = new SerialTool());
        }

        public List<string> FindPort()
        {
            var portList = new List<string>();
            for (var i = 0; i < 16; i++)
            {
                try
                {
                    var serialPort = new SerialPort("COM" + (i + 1).ToString());
                    serialPort.Open();
                    serialPort.Close();
                    portList.Add("COM" + (i + 1).ToString());
                }
                catch (Exception)
                {
                    continue;
                }
            }
            return portList;
        }

        public delegate void SerialListener(object sender, SerialDataReceivedEventArgs e);

        public SerialPort OpenPort(string portName, string baudRate, string dataBits, string stopBits, string parity, SerialListener serialListener)
        {
            // 检测串口设置
            if (portName == "" || baudRate == "" || dataBits == "" || stopBits == "" || parity == "")
            {
                // MessageBox.Show("串口未设置！", "错误提示");
                return null;
            }
            // 设置串口
            var serialPort = SetPortProperty(portName, baudRate, dataBits, stopBits, parity, serialListener);
            // 打开串口
            try
            {
                serialPort.Open();
                return serialPort;
            }
            catch (Exception)
            {
                // MessageBox.Show("串口无效或已被占用！", "错误提示");
                return null;
            }
        }

        private SerialPort SetPortProperty(string portName, string baudRate, string dataBits, string stopBits, string parity, SerialListener serialListener)
        {
            // 设置串口的属性
            var serialPort = new SerialPort();
            // 设置串口名
            serialPort.PortName = portName.Trim();
            // 设置串口的波特率
            serialPort.BaudRate = Convert.ToInt32(baudRate.Trim());
            // 设置停止位
            float f = Convert.ToSingle(stopBits.Trim());
            if (f == 0)
            {
                serialPort.StopBits = StopBits.None;
            }
            else if (f == 1.5)
            {
                serialPort.StopBits = StopBits.OnePointFive;
            }
            else if (f == 1)
            {
                serialPort.StopBits = StopBits.One;
            }
            else if (f == 2)
            {
                serialPort.StopBits = StopBits.Two;
            }
            else
            {
                serialPort.StopBits = StopBits.One;
            }
            // 设置数据位 
            serialPort.DataBits = Convert.ToInt16(dataBits.Trim());
            // 设置奇偶校验位
            string s = parity.Trim();
            if (String.Compare(s, "无", StringComparison.Ordinal) == 0)
            {
                serialPort.Parity = Parity.None;
            }
            else if (String.Compare(s, "奇校验", StringComparison.Ordinal) == 0)
            {
                serialPort.Parity = Parity.Odd;
            }
            else if (s.CompareTo("偶校验") == 0)
            {
                serialPort.Parity = Parity.Even;
            }
            else
            {
                serialPort.Parity = Parity.None;
            }
            // 设置超时读取时间
            serialPort.ReadTimeout = -1;
            serialPort.RtsEnable = true;
            // 定义 DataReceived 事件，当串口收到数据后触发事件
            serialPort.DataReceived += new SerialDataReceivedEventHandler(serialListener);
            return serialPort;
        }

        public bool ClosePort(SerialPort serialPort)
        {
            // 关闭串口
            try
            {
                serialPort.Close();
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }

        public bool SendToPort(SerialPort serialPort, string data)
        {
            // 写串口数据
            if (serialPort.IsOpen)
            {
                try
                {
                    serialPort.WriteLine(data);
                    return true;
                }
                catch (Exception)
                {
                    // MessageBox.Show("发送数据时发生错误！", "错误提示");
                    return false;
                }
            }
            else
            {
                // MessageBox.Show("串口未打开！", "错误提示");
                return false;
            }
        }
    }
}
